@@ -199,7 +199,8 @@ get_package_for_basename (const gchar *basename)
 		get_hv ("Glib::Object::Introspection::_BASENAME_TO_PACKAGE", 0);
 	g_assert (basename_to_package);
 	svp = hv_fetch (basename_to_package, basename, strlen (basename), 0);
-	g_assert (svp && gperl_sv_is_defined (*svp));
+	if (!svp || !gperl_sv_is_defined (*svp))
+	    return NULL;
 	return SvPV_nolen (*svp);
 }
 
