@@ -6,19 +6,22 @@ sv_to_callback (GIArgInfo * arg_info,
                 SV * sv,
                 GPerlI11nInvocationInfo * invocation_info)
 {
+	GIBaseInfo *callback_interface_info;
 	GPerlI11nPerlCallbackInfo *callback_info;
 	GIScopeType scope;
 
-	/* the destroy notify func is handled by handle_automatic_arg */
+	/* the destroy notify func is handled by _handle_automatic_arg */
 
 	dwarn ("      Perl callback at %d (%s)\n",
 	       invocation_info->current_pos,
 	       g_base_info_get_name (arg_info));
 
-	callback_info = create_perl_callback_closure (type_info, sv);
+	callback_interface_info = g_type_info_get_interface (type_info);
+	callback_info = create_perl_callback_closure (callback_interface_info, sv);
 	callback_info->data_pos = g_arg_info_get_closure (arg_info);
 	callback_info->destroy_pos = g_arg_info_get_destroy (arg_info);
 	callback_info->free_after_use = FALSE;
+	g_base_info_unref (callback_interface_info);
 
 	dwarn ("      Perl callback data at %d, destroy at %d\n",
 	       callback_info->data_pos, callback_info->destroy_pos);
@@ -29,9 +32,8 @@ sv_to_callback (GIArgInfo * arg_info,
 	switch (scope) {
 	    case GI_SCOPE_TYPE_CALL:
 		dwarn ("      Perl callback has scope 'call'\n");
-		invocation_info->free_after_call
-			= g_slist_prepend (invocation_info->free_after_call,
-			                   callback_info);
+		free_after_call (invocation_info,
+		                 (GFunc) release_perl_callback, callback_info);
 		break;
 	    case GI_SCOPE_TYPE_NOTIFIED:
 		dwarn ("      Perl callback has scope 'notified'\n");
@@ -91,7 +93,7 @@ callback_to_sv (GICallableInfo *interface, gpointer func, GPerlI11nInvocationInf
 	GSList *l;
 	for (l = invocation_info->callback_infos; l != NULL; l = l->next) {
 		GPerlI11nCCallbackInfo *callback_info = l->data;
-		if (invocation_info->current_pos == callback_info->destroy_pos) {
+		if ((gint) invocation_info->current_pos == callback_info->destroy_pos) {
 			dwarn ("      destroy notify for C callback %p\n",
 			       callback_info);
 			callback_info->destroy = func;
@@ -144,7 +146,7 @@ callback_data_to_sv (gpointer data,
 		return NULL;
 	for (l = invocation_info->callback_infos; l != NULL; l = l->next) {
 		GPerlI11nCCallbackInfo *callback_info = l->data;
-		if (callback_info->data_pos == invocation_info->current_pos) {
+		if (callback_info->data_pos == (gint) invocation_info->current_pos) {
 			dwarn ("      user data for C callback %p\n",
 			       callback_info);
 			attach_c_callback_data (callback_info, data);
